/*========================================================================*
 * TITLE:	MGS - Motif Good Stuff
 * MODULE:	mgs.c
 * BY:		Ross C Linder (c) 1995
 *		ross@mecalc.co.za
 * REQUIRES:	Motif 2.0 (Xpm Extensions)
 *
 * This program is free software; permission hereby granted, free of
 * charge, to any person obtaining a copy of this software and 
 * associated documentations to copy, modify and  distribute it under 
 * the terms of the GNU General Public License as published by 
 * the Free Software Foundation, subject to the following conditions: 
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
 * FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
 * OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
 * TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
 * PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION.
 *========================================================================*/

static char ID[] =

"@(#) mgs.c	$Revision: 1.2 $	$Date: 1995/11/27 09:25:34 $";

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <X11/IntrinsicP.h>
#include <X11/CompositeP.h>
#include <Xm/XmP.h>
#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>

#define MAX_ICONS		50


XtTimerCallbackProc test_icons ();

Widget	icon_pb[MAX_ICONS];
Widget	icon_tab[MAX_ICONS];
Widget	icon_lab[MAX_ICONS];
char	*cmd_tab[MAX_ICONS];
char	*cmd_des[MAX_ICONS];
char	*cmd_icf[MAX_ICONS];

Display		*disp;
XtAppContext	app_context;
Widget		toplevel;
Widget		rbox;
char		*app_name;
Pixel		foreground, background;

void set_icon_image (char *icf);

/*=================== Resource database structures ====================*/

static  struct  iv {
	String	columns;
	} iv;

static XtResource resources[] = {
	    {
	    "NumColumns",	XtCLabel,
	    XtRString, sizeof (String),
	    0,
	    XtRString, "15"
	    }
	};

/*========================================================================*
 * MAIN -- Create the app shell and RC, start the timer
 *========================================================================*/

main (int argc, char *argv[])
{
Widget		form, twid;
XmString	xms;
Arg		args[10];
int		a;
int		ncol = 0;

    if ((app_name = strrchr (argv[0], '/')) != NULL)
	app_name++;
    else
	app_name = argv[0];

    for (a = 0; a < MAX_ICONS; a++) {
	icon_tab[a] = NULL;
	cmd_tab[a] = NULL;
	}

    XtToolkitInitialize();

    app_context = XtCreateApplicationContext ();
    disp = XtOpenDisplay(app_context, NULL, argv[0], app_name, NULL,
						0, &argc, argv);

    a = 0;
    XtSetArg(args[a], XmNallowShellResize, True);  a++;
    toplevel = XtAppCreateShell(argv[0], app_name,
		applicationShellWidgetClass, disp, args,a);

    XtGetApplicationResources (toplevel, &iv, resources, 1, NULL, 0);

    ncol = atoi (iv.columns);

    if (ncol <= 0)
	ncol = 15;

    form = XmCreateForm (toplevel, "form", NULL, 0);
    XtManageChild (form);
    set_icon_image ("/usr/lib/X11/mgs/mgs.xpm");

    a = 0;
    XtSetArg (args[a], XmNnumColumns, ncol); a++;
    XtSetArg (args[a], XmNpacking, XmPACK_COLUMN); a++;
    XtSetArg (args[a], XmNentryVerticalAlignment, XmALIGNMENT_CENTER); a++;
    XtSetArg (args[a], XmNadjustLast, False); a++;
    rbox = XmCreateRowColumn (form, "rbox", args, a);
    XtManageChild (rbox);

    a = 0;
    XtSetArg (args[a], XmNbackground, &background); a++;
    XtSetArg (args[a], XmNforeground, &foreground); a++;
    XtGetValues (rbox, args, a);

    XtAppAddTimeOut (app_context, 1000, (XtTimerCallbackProc)test_icons, 0);

    XtRealizeWidget (toplevel);

    XtAppMainLoop (app_context);
}


/*========================================================================*
 * DOIT -- Execute the requested command, I use system as it is easy and
 *         with dll's not much overhead at all.
 *========================================================================*/

void doit (Widget w, int index)
{
char		cmd[256];

    strcpy (cmd, cmd_tab[index]);
    strcat (cmd, " &");

    system (cmd);
}


/*========================================================================*
 * TEST_ICONS -- Check for new entries in the config file
 *========================================================================*/

XtTimerCallbackProc test_icons ()
{
static		mtime = 0;
struct stat	sb;
int		a, b, c;
Pixmap		pm, opm;
XmString	xms;
Arg		args[5];
char		buf[256], *bp;
char		icf[256], cname[32];
FILE		*fp;
Widget		twid;

    XtAppAddTimeOut (app_context, 1000, (XtTimerCallbackProc)test_icons, 0);

    strcpy (icf, getenv ("HOME"));
    strcat (icf, "/.");
    strcat (icf, app_name);

    if (stat (icf, &sb) == -1) {
	strcpy (icf, "/usr/lib/X11/mgs/");
	strcat (icf, app_name);
	if (stat (icf, &sb) == -1)
	    return;
	}

    if (mtime == sb.st_mtime)
	return;

    mtime = sb.st_mtime;

    if ((fp = fopen (icf, "r")) != NULL) {
	c = 0;
	for (;;) {
	    opm = 0;
	    a = getc (fp);
	    if (a == '\n')		/* Skip blank lines */
		continue;

	    ungetc (a, fp);
	    if (a == '#') {		/* and comments */
		fgets (buf, 256, fp);
		continue;
		}
	    if (fscanf (fp, "%[\040-\176]\t%[\040-\176]\t%[\040-\176]\n",
			cname, buf, icf) != 3)
		break;

	    if ((bp = strrchr (buf, '/')) != NULL)
		bp ++;
	    else
		bp = buf;

	    if (icon_tab[c] != NULL) {
		if (strcmp (cmd_tab[c], cname) != 0 ||
			strcmp (cmd_des[c], cname) != 0 ||
			strcmp (cmd_icf[c], icf) != 0) {
		    free (cmd_tab[c]);
		    free (cmd_des[c]);
		    free (cmd_icf[c]);

		    XtSetArg (args[0], XmNlabelPixmap, &opm);
		    XtGetValues (icon_pb[c], args, 1);
		    }
		else {
		    c++;
		    continue;
		    }
		}
	    else {
		b = 0;
		XtSetArg (args[b], XmNshadowType, XmSHADOW_ETCHED_OUT); b++;
		icon_tab[c] = XmCreateFrame (rbox, "pbf", args, b);
		XtManageChild (icon_tab[c]);
		
		b = 0;
		XtSetArg (args[b], XmNchildType, XmFRAME_WORKAREA_CHILD); b++;
		XtSetArg (args[b], XmNbottomAttachment, XmATTACH_WIDGET); b++;
		XtSetArg (args[b], XmNbottomWidget, icon_tab[c]); b++;
		icon_pb[c] = XmCreatePushButton (icon_tab[c], "pb", args, b);
		XtManageChild (icon_pb[c]);

		XtAddCallback (icon_pb[c], XmNactivateCallback,
			(XtCallbackProc)doit, c);

		b = 0;
		xms = XmStringCreateSimple ("?");
		XtSetArg (args[b], XmNchildType, XmFRAME_TITLE_CHILD); b++;
		XtSetArg (args[b], XmNchildHorizontalAlignment,
						XmALIGNMENT_CENTER); b++;
		XtSetArg (args[b], XmNchildVerticalAlignment,
						XmALIGNMENT_CHILD_TOP); b++;
		XtSetArg (args[b], XmNlabelString, xms); b++;
		icon_lab[c] = XmCreateLabel (icon_tab[c], "pbfl", args, b);
		XtManageChild (icon_lab[c]);
		XmStringFree (xms);
		}

	    b = 0;
	    xms = XmStringCreateSimple (cname);
	    XtSetArg (args[b], XmNlabelString, xms); b++;
	    XtSetValues (icon_lab[c], args, b);
	    XmStringFree (xms);

	    pm = XmGetPixmap (XtScreen (toplevel), icf, foreground, background);

	    if (pm == XmUNSPECIFIED_PIXMAP)
		pm = XmGetPixmap (XtScreen (toplevel),
		    "/usr/lib/X11/mgs/none.xpm", foreground, background);


	    xms = XmStringCreateSimple (bp);

	    b = 0;
	    if (pm != XmUNSPECIFIED_PIXMAP) {
		XtSetArg (args[b], XmNlabelType, XmPIXMAP); b++;
		XtSetArg (args[b], XmNlabelPixmap, pm); b++;
		}
	    else {
		XtSetArg (args[b], XmNlabelString, xms); b++;
		}

	    XtSetArg (args[b], XmNalignment, XmALIGNMENT_CENTER); b++;
	    XtSetValues (icon_pb[c], args, b);

	    XmStringFree (xms);

	    cmd_tab[c] = strdup (buf);
	    cmd_des[c] = strdup (cname);
	    cmd_icf[c] = strdup (icf);
	    c++;

	    if (opm)
		XmDestroyPixmap (XtScreen (toplevel), opm);
	    }
	fclose (fp);
	}
}

/*========================================================================*
 * SET_ICON_IMAGE -- Create an Icon window with a pixmap
 *========================================================================*/

void set_icon_image (char *icf)
{
Pixmap		pm;
Window		icon;
struct stat	sb;

    
    if (stat (icf, &sb) == -1)		/* No Pixmap file */
	return;

    pm = XmGetPixmap (XtScreen (toplevel), icf, foreground, background);

    icon = XCreateSimpleWindow (disp,
		XRootWindowOfScreen (XtScreen (toplevel)),
		0, 0, 64, 64, 8,
		foreground,
		background);

    XSetWindowBackgroundPixmap (disp, icon, pm);
    XtVaSetValues (toplevel, XtNiconWindow, icon, NULL);
}

/*========================================================================*
 * $Log: mgs.c,v $
 * Revision 1.2  1995/11/27  09:25:34  ross
 * Improve config file change tests
 *
 * Revision 1.1  1995/11/23  18:09:10  ross
 * Initial revision
 *
 *========================================================================*/
